//this file is part of eMule
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
#include "stdafx.h"
#include <winsock2.h>
#include "ED2KLink.h"
#include "otherfunctions.h"
#include "emule.h"

namespace {
	struct autoFree {
		autoFree(TCHAR* p) : m_p(p) {}
		~autoFree() { free(m_p); }
	private:
		TCHAR * m_p;
	};
	inline unsigned int FromHexDigit(TCHAR digit) {
		switch (digit) {
		case _T('0'): return 0;
		case _T('1'): return 1;
		case _T('2'): return 2;
		case _T('3'): return 3;
		case _T('4'): return 4;
		case _T('5'): return 5;
		case _T('6'): return 6;
		case _T('7'): return 7;
		case _T('8'): return 8;
		case _T('9'): return 9;
		case _T('A'): return 10;
		case _T('B'): return 11;
		case _T('C'): return 12;
		case _T('D'): return 13;
		case _T('E'): return 14;
		case _T('F'): return 15;
		case _T('a'): return 10;
		case _T('b'): return 11;
		case _T('c'): return 12;
		case _T('d'): return 13;
		case _T('e'): return 14;
		case _T('f'): return 15;
		default: throw GetResString(IDS_ERR_ILLFORMEDHASH);
		}
	}
}

CED2KLink::~CED2KLink()
{
}

///////////////////////////////////////////// 
// CED2KServerListLink implementation 
///////////////////////////////////////////// 
CED2KServerListLink::CED2KServerListLink(const TCHAR* address)
{
	m_address = address;
}

CED2KServerListLink::~CED2KServerListLink()
{
} 

void
CED2KServerListLink::GetLink(CString& lnk)
{
	lnk = _T("ed2k://|serverlist|");
	lnk += m_address;
	lnk += _T("|/");
}

CED2KServerListLink*
CED2KServerListLink::GetServerListLink()
{
	return this;
}

CED2KServerLink*
CED2KServerListLink::GetServerLink()
{
	return 0;
}

CED2KFileLink*
CED2KServerListLink::GetFileLink()
{
	return 0;
}

CED2KLink::LinkType
CED2KServerListLink::GetKind() const
{
	return kServerList;
}

/////////////////////////////////////////////
// CED2KServerLink implementation
/////////////////////////////////////////////
CED2KServerLink::CED2KServerLink(const TCHAR* ip,const TCHAR* port)
{
	m_ip = inet_addr(ip);
	unsigned long ul = _tcstoul(port,0,10);
	if ( ul > 0xFFFF )
		throw GetResString(IDS_ERR_BADPORT);
	m_port = static_cast<uint16>(ul);
	m_defaultName = "Server ";
	m_defaultName += ip;
	m_defaultName += ":";
	m_defaultName += port;
}

CED2KServerLink::~CED2KServerLink()
{
}


void 
CED2KServerLink::GetLink(CString& lnk)
{
	in_addr adr;
	char buffer[32];
	lnk = _T("ed2k://|server|");
	adr.S_un.S_addr = m_ip;
	lnk += inet_ntoa(adr);
	lnk += _T("|");
	sprintf(buffer,"%d",static_cast<int>(m_port));
	lnk += buffer;
	lnk += _T("|/");
}

CED2KServerListLink*
CED2KServerLink::GetServerListLink() 
{ 
	return 0; 
}

CED2KServerLink* 
CED2KServerLink::GetServerLink() 
{ 
	return this; 
}

CED2KFileLink* 
CED2KServerLink::GetFileLink() 
{ 
	return 0; 
}

CED2KLink::LinkType 
CED2KServerLink::GetKind() const
{
	return kServer;
}


/////////////////////////////////////////////
// CED2KFileLink implementation
/////////////////////////////////////////////
CED2KFileLink::CED2KFileLink(const TCHAR* name,const TCHAR* size, const TCHAR* hash, const TCHAR* sources)
: m_name(name)
, m_size(size)
{
	if ( _tcslen(hash) != 32 )
		throw GetResString(IDS_ERR_ILLFORMEDHASH);
	for ( int idx = 0 ; idx < 16 ; ++idx) {
		m_hash[idx] = FromHexDigit(*hash++)*16;
		m_hash[idx] += FromHexDigit(*hash++);
	}
	SourcesList=NULL;

	if (sources){
		CString strSources;
		int nSrcOffset;
		int nSrcEnd;
		bool bHasExpiration;
		bool bExpirated;
		int nYear,nMonth,nDay;
		strSources.Append(sources);
		nSrcOffset = strSources.Find("sources",0); 
		if(nSrcOffset >= 0){
			nSrcOffset=7;
			nSrcEnd = strSources.Find('|',nSrcOffset);
			if (nSrcEnd=-1) nSrcEnd=strSources.GetLength();
			bHasExpiration=false;
			bExpirated=false;
			if(sources[nSrcOffset] == '@' ) {
					CString strTemp;
					COleDateTime expirationDate;
					nSrcOffset++;
					strTemp = strSources.Mid(nSrcOffset,2);
					nYear = atoi((LPCTSTR)strTemp) + 2000;
					nSrcOffset = nSrcOffset+2;
					strTemp = strSources.Mid(nSrcOffset,2);
					nMonth = atoi((LPCTSTR)strTemp);
					nSrcOffset = nSrcOffset+2;
					strTemp = strSources.Mid(nSrcOffset,2);
					nDay = atoi((LPCTSTR)strTemp);
					nSrcOffset = nSrcOffset+2;
					bHasExpiration=(expirationDate.SetDate(nYear,nMonth,nDay) == 0);
					if (bHasExpiration) bExpirated=(COleDateTime::GetCurrentTime() > expirationDate);
			}
			if (!bExpirated||!bHasExpiration) {
				if( nSrcEnd > nSrcOffset && (sources[nSrcOffset] == ',' ))
					strSources = strSources.Mid( nSrcOffset, nSrcEnd - nSrcOffset );
				if( strSources.GetLength() > 5 ) {
					int nCurPos=0;
					CString token;
					unsigned long ul;
					uint16 nCount=0;
					uint32 dwID;
					uint16 nPort;

					uint32 dwServerIP = 0; 
					uint16 nServerPort = 0;
					SourcesList=new CMemFile();
					SourcesList->Write(&nCount,2); 

					token = strSources.Tokenize(",:",nCurPos);
					while ( (token != "") && (nCount<0xFFFF)) {
						dwID = inet_addr((LPCTSTR)token);
						token = strSources.Tokenize(",:",nCurPos);
						if (token!=""){
							ul = _tcstoul((LPCTSTR)token,0,10);
							nPort = static_cast<uint16>(ul);

							token = strSources.Tokenize(",:",nCurPos);
							// skip bad ips / ports
							if( dwID == INADDR_NONE || dwID < 16777216 || 
								ul > 0xFFFF || ul == 0 )
								{ continue; }
							SourcesList->Write(&dwID,4);
							SourcesList->Write(&nPort,2);
							SourcesList->Write(&dwServerIP,4);
							SourcesList->Write(&nServerPort,2);
							nCount++;
						}
					}
					SourcesList->SeekToBegin();
					SourcesList->Write(&nCount,sizeof(nCount));
					SourcesList->SeekToBegin();
					if (nCount==0) {
						delete SourcesList;
						SourcesList=NULL;
					}
				}
			}
		}
	}
}


CED2KFileLink::~CED2KFileLink()
{
	if (SourcesList){
		delete SourcesList;
		SourcesList=NULL;
	}
}

void 
CED2KFileLink::GetLink(CString& lnk)
{
	lnk = _T("ed2k://|file|");
	lnk += m_name;
	lnk += _T("|");
	lnk += m_size;
	lnk += _T("|");
	for (int idx=0; idx != 16 ; ++idx ) {
		unsigned int ui1 = m_hash[idx] / 16;
		unsigned int ui2 = m_hash[idx] % 16;
		lnk+= static_cast<TCHAR>( ui1 > 9 ? (_T('0')+ui1) : (_T('A')+(ui1-10)) );
		lnk+= static_cast<TCHAR>( ui2 > 9 ? (_T('0')+ui2) : (_T('A')+(ui2-10)) );
	}
	lnk += _T("|/");
}

CED2KServerListLink*
CED2KFileLink::GetServerListLink() 
{ 
	return 0; 
}

CED2KServerLink* 
CED2KFileLink::GetServerLink() 
{ 
	return 0; 
}
CED2KFileLink* 
CED2KFileLink::GetFileLink() 
{ 
	m_name.Replace("%20"," ");
	return this; 
}

CED2KLink::LinkType 
CED2KFileLink::GetKind() const
{
	return kFile;
}


//static 
CED2KLink* 
CED2KLink::CreateLinkFromUrl( const TCHAR * uri)
{
	// Parse pseudo-URI
	const TCHAR* pChArray[7];
	if (uri==0) 
		throw CString(_T("null ed2k link"));
	TCHAR* pNewString = _tcsdup(uri);
	autoFree liberator(pNewString);
	TCHAR* pCh = pNewString;
	const TCHAR* pStart = pCh;
	int idx = 0;
	for (idx=0;idx<7;idx++) pChArray[idx]=NULL;
	idx = 0;
	while( idx <7 && ((pCh = _tcschr(pCh,_T('|'))) !=0) ) {
		pChArray[idx++] = pStart;
		*pCh = 0;
		++ pCh;
		pStart = pCh;
	}
	if ( *pStart != _T('/') ) {
		throw CString(_T(GetResString(IDS_ERR_BADED2KLINK)));
	}
	if (   idx < 3
		|| pChArray[0] == 0 
		|| pChArray[1] == 0 
		|| pChArray[2] == 0 
//		|| pChArray[3] == 0 // This was preventing ed2k serverlist links from working.. 
		|| _tcscmp( _T("ed2k://") , pChArray[0]  ) != 0 
	   ) {
			throw GetResString(IDS_ERR_BADED2KLINK);
	}
	if ( _tcscmp( _T("file") , pChArray[1]  ) == 0 && idx >=  5 && pChArray[4] != 0 ) {
		return new CED2KFileLink(pChArray[2],pChArray[3],pChArray[4],pChArray[6]);
	}
	else if ( _tcscmp( _T("serverlist") , pChArray[1] ) == 0 && idx == 3 ) {
		return new CED2KServerListLink(pChArray[2]);
	}
	else if ( _tcscmp( _T("server") , pChArray[1]  ) == 0 && idx == 4 ) {
		return new CED2KServerLink(pChArray[2],pChArray[3]);
	}
	else {
		throw GetResString(IDS_ERR_NOSLLINK);
	}
	return 0;
}
